package com.company.jedr.view.customer;

import com.company.jedr.entity.Customer;
import com.company.jedr.entity.Order;
import com.company.jedr.view.main.MainView;
import com.vaadin.flow.component.AttachEvent;
import com.vaadin.flow.component.html.Span;
import com.vaadin.flow.router.Route;
import com.vaadin.flow.router.RouteParameters;
import io.jmix.flowui.ViewNavigators;
import io.jmix.flowui.view.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;

@Route(value = "customers/:id", layout = MainView.class)
@ViewController("Customer.detail")
@ViewDescriptor("customer-detail-view.xml")
@EditedEntityContainer("customerDc")
public class CustomerDetailView extends StandardDetailView<Customer> {
    private static final Logger log = LoggerFactory.getLogger(CustomerDetailView.class);
    @ViewComponent
    private Span totalAmountSpan;
    @Autowired
    private ViewNavigators viewNavigators;

    @Subscribe
    public void onAttachEvent(final AttachEvent event) {
        if (CollectionUtils.isEmpty(getEditedEntity().getOrders())) {
            return;
        }

        getEditedEntity().getOrders().stream()
                .map(Order::getAmount).reduce(Double::sum)
                .ifPresent(totalAmount -> totalAmountSpan.setText("Total amount: " + totalAmount));
    }

    @Install(to = "ordersDataGrid.create", subject = "afterSaveHandler")
    private void ordersDataGridCreateAfterSaveHandler(final Order order) {
        closeWithSave()
                .then(() -> viewNavigators.detailView(Customer.class)
                        .withViewClass(CustomerDetailView.class)
                        .withRouteParameters(new RouteParameters("id", getEditedEntity().getId().toString()))
                        .navigate()
                );
    }

    @Subscribe
    public void onAfterClose(final AfterCloseEvent event) {
        log.info("Close Event: " + event.getCloseAction());
    }



}
